import json
import yaml
import logging
import coloredlogs

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


class Yaml:
    def __init__(self, yaml_path):
        self.yaml_path = yaml_path

    def read_provision_yaml(self) -> json:
        """ Load YAML configuration file to a dictionary
        """
        with open(self.yaml_path, 'r') as file:
            parsed_yaml = yaml.safe_load(file)
            logger.debug(parsed_yaml)
            return parsed_yaml


# x = Yaml(yaml_path='/usr/local/google/home/hugoselbie/code_sample/py/clients/snap/okta_groups/snap_group_mapping.yaml')
# print(x.read_provision_yaml())



#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
from lmanage import get_content_with_views
from lmanage import provision_instance_permission_structure
import coloredlogs
import logging
from lmanage.utils import create_df

logger = logging.getLogger(__name__)


@click.group()
@click.version_option()
def lmanage():
    pass


@lmanage.command()
@click.option("-fp", "--path",
              help="input your file path to save a csv of results")
@ click.option("-i", "--ini-file",
               help="Path to the ini file to use for sdk authentication")
@ click.option("-p", "--project",
               help="Path folder containing your lookml files, often taken using a git pull from your connected lookml project repository")
@ click.option("-t", "--table",
               help="**OPTIONAL** Add a view name to search for elements that rely on this view")
@ click.option("-f", "--field",
               help="**OPTIONAL** Add a fully scoped fieldname (e.g. view_name.field_name) to return a csv with these values")
@ click.option("-l", "--level",
               default='INFO',
               help="**OPTIONAL** Add the value 'DEBUG' to get a more verbose version of the returned stout text")
def mapview(**kwargs):
    level = kwargs.get('level', 'INFO')
    coloredlogs.install(level=level, logger=logger)
    for k, v in kwargs.items():
        if {v} != None:
            logger.info(
                f'You have set {v} for your {k} variable')
        else:
            logger.debug(
                f'There is no value set for {k} please use the `--help` flag to see input parameters')
    get_content_with_views.main(**kwargs)


@lmanage.command()
@ click.option("-i", "--ini-file",
               help="**OPTIONAL ** Specify API Credentials in an ini file, if no path is given program will assume these values are set as environmental variables as denoted at https: // github.com/looker-open-source/sdk-codegen  # environment-variable-configuration")
@ click.option("-yp", "--yaml-config-path",
               help="Path to the yaml file to use for instance configuration")
@ click.option("-l", "--level",
               default='INFO',
               help="**OPTIONAL** Add the value 'DEBUG' to get a more verbose version of the returned stout text")
def configurator(**kwargs):
    level = kwargs.get('level', 'INFO')
    coloredlogs.install(level=level, logger=logger)
    for k, v in kwargs.items():
        if {v} != None:
            logger.info(
                f'You have set {v} for your {k} variable')
        else:
            logger.debug(
                f'There is no value set for {k} please use the `--help` flag to see input parameters')
    provision_instance_permission_structure.main(**kwargs)


# @instance_config.command()
# @click.option("-t", "--test")
# def update_configs(**kwargs):
#     print(kwargs.items())


# @instance_config.command()
# @click.option("-t", "--test")
# def user_permissions(**kwargs):
#     print('users')


# @instance_config.command()
# @click.option("-t", "--test")
# def full_instance_config(**kwargs):
#     print('full')



#!/usr/bin/python
#
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import looker_sdk
from looker_sdk import models
import json
from pathlib import Path
from collections import defaultdict
import pandas as pd
import lookml
import configparser as ConfigParser
import snoop
import re
from itertools import chain
from lmanage.utils import parsing_sql
from lmanage.utils import create_df
import coloredlogs
import logging
import sys

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logging.getLogger().addHandler(logging.StreamHandler(sys.stdout))
fh = logging.FileHandler('map_view.log')
fh.setLevel(logging.DEBUG)
formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
fh.setFormatter(formatter)
logger.addHandler(fh)

logger = logging.getLogger(__name__)


def find_model_files(proj):
    """Fetches model files from PyLookML obj.
    Scans all files in PyLookML object and returns a file if that file has a type of Model.

    Args:
        proj: The project from PyLookML
    Returns:
        The unparsed model file of a project.
    """
    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type == 'model':
            return file


def get_view_path(proj):
    """Fetches the file path of a PyLookML Oject.

    Iterates over a list of PyLookML files and returns a list of the files if they are of type 'View'
    Args:
        proj: The project from PyLookML
    Returns:
        A list of all the file paths pointing towards view objects that are kept in a PyLookML object.
        For example:
        ['proj/views/order_items.view.lkml','proj/views/inventory_items.view.lkml','proj/views/products.view.lkml']
    """
    view_list = defaultdict(list)
    for file in proj.files():
        path = file.path
        path = path.split('.')
        if path[-2] == 'view':
            name = file.name.split('.')
            view_list[name[0]].append(file.path)

    return view_list


def fetch_view_files(proj):
    """Fetches the all the named view objects listed in explores of a Model file specifically.
    Identifies a model file, iterates over all the explore objects in the model file, appends their base view and joined views to a dict object
    Args:
        proj: The project from PyLookML
    Returns:
        A dict with key: being the base view and values: list of all the used views in the explore object.
        For example:
         defaultdict(<class 'list'>, {'order_items': ['order_items', 'order_facts', 'inventory_items', 'users', 'user_order_facts', 'products', 'repeat_purchase_facts', 'distribution_centers', 'test_ndt'], 'events':['events', 'sessions', 'session_landing_page', 'session_bounce_page', 'product_viewed', 'users', 'user_order_facts']})    """

    true_view_names = defaultdict(list)

    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type == 'model':
            my_model = file

            for explore in my_model.explores:
                if 'view_name' not in explore and 'from' not in explore:
                    true_view_names[explore.name].append(explore.name)
                if 'view_name' in explore:
                    true_view_names[explore['name']].append(
                        explore['view_name'].value)
                if 'from' in explore:
                    true_view_names[explore['name']].append(
                        explore['from'].value)
                if 'join' in explore:
                    for join in explore['join']:
                        true_view_names[explore['name']].append(join['name'])
                        if 'view_name' in join.__dict__.keys():
                            true_view_names[explore['name']].append(
                                join['view_name'].value)
                        if 'from' in join.__dict__.keys():
                            true_view_names[explore['name']].append(
                                join['from'].value)
    return true_view_names


def get_sql_table_name(proj):
    """Fetches the sql_table_name of view files listed in a PyLookML Oject.

    Iterates over a list of PyLookML files identifies view files aand returns a list of the sql_table_names if they exist in an object of type 'View'
    Args:
        proj: The project from PyLookML
    Returns:
        A list of all the sql_table_names that are found in a series of view objects.
        For example:
        ['public.order_items','public.inventory_items','public.events']
    """
    view_list = defaultdict(list)

    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type == 'partial_model':
            for view in myFile.views:
                if view.sql_table_name.value:
                    view_list[path].append(view.sql_table_name.value)

    return view_list


def get_sql_table_name_list(sql_table_name_dict, key: True):
    response = []
    for k, v in sql_table_name_dict.items():
        if key:
            response.append(k)
        else:
            response.append(v)

    return response if key else list(chain(*response))


def parse_sql(sdk, qid: int):
    """Idenfies the base tables and joins used by a Looker query.

    Iterates over a list of PyLookML files identifies view files aand returns a list of the sql_table_names if they exist in an object of type 'View'
        qid: (int) query_id from a  Looker query (n.b. NOT THE SAME AS A QID in the url)
    Returns:
        A list of all the tables that are found from a Looker generated SQL query.
        For example:
        ['public.order_items','public.inventory_items','public.events']
    Exception:
        If a query is broken for whatever reason an Exception is raised to continue the program running
    """
    try:
        sql_response = sdk.run_query(query_id=qid, result_format='sql')
        if type(sql_response) == str:
            tables = parsing_sql.extract_tables(sql_response)
            return tables
        else:
            return sql_response
    except looker_sdk.error.SDKError:
        return('No Content')


def get_sql_from_elements(sdk, content_results):
    """Amends returned SDK System__Activity reponse with sql tables used from the `parse_sql` function.

    Iterates over the response from get_dashboards and runs the parse_sql function for each returned dashboard element, returns the list of tables and amends the dict response and returns it
    Args:
        sdk: Looker SDK object
        content_results: (dict) response from get_dashboards function call
    Returns:
        An amended dict response with the sql columns used by each element extracted our of the Looker generated SQL for each dashboard object.
        For example:
    [{'dashboard.id': 1,
         'dashboard_element.id': 1,
         'dashboard_element.type': 'vis',
         'dashboard_element.result_source': 'Lookless',
         'query.model': 'bq',
         'query.view': 'order_items',
         'query.formatted_fields': '["order_items.created_month", "order_items.count"]',
         'query.id': 59,
         'dashboard.title': 'dash_1',
         'look.id': None,
         'sql_joins': ['`looker-private-demo.ecomm.order_items`']}]
    """
    for dash in content_results:
        query_id = dash['query.id']
        sql_value = parse_sql(sdk, query_id)

        dash['sql_joins'] = sql_value

    return content_results


def get_dashboards(sdk):
    """Uses the Looker SDK System__Activity model to extract dashboard and dashboard_element metadata.

    Simple run_inline_query call to Looker SDK
    Args:
        sdk: Looker SDK object
    Returns:
        An dict response with the dashboard and dashboard_element metadata.
        For example:
    [{'dashboard.id': 1,
         'dashboard_element.id': 1,
         'dashboard_element.type': 'vis',
         'dashboard_element.result_source': 'Lookless',
         'query.model': 'bq',
         'query.view': 'order_items',
         'query.formatted_fields': '["order_items.created_month", "order_items.count"]',
         'query.id': 59,
         'dashboard.title': 'dash_1',
         'look.id': None}]
    """
    query_config = models.WriteQuery(
        model="system__activity",
        view="dashboard",
        fields=[
            "dashboard.id",
            "dashboard_element.id",
            "dashboard_element.type",
            "dashboard_element.result_source",
            "query.model",
            "query.view",
            "query.formatted_fields",
            "query.id",
            "dashboard.title",
            "look.id"
        ],
        filters={
            "dashboard_element.type": "-text",
            "dashboard.deleted_date": "NULL"
        },
        limit='5000'
    )
    query_response = sdk.run_inline_query(
        result_format='json',
        body=query_config
    )

    query_response = json.loads(query_response)

    return query_response


def test_period_appearence(input_response):
    """Checks existence of a period in an input string.

    Simple return True if a period is detected in a string
    Args:
        input_response: (str)
    Returns:
        A boolean response
    For example:
    True or False
    """
    test_period = re.search(r"\.", input_response)
    return bool(test_period)


def match_join_per_query(myresults):
    result = []
    sql_join = myresults['sql_joins']
    sql_table_name = myresults['sql_table_name']

    for sql in sql_join:
        if not bool(test_period_appearence(sql)):
            result.append(sql)
        for name in sql_table_name:
            if sql == name:
                result.append(sql)
    myresults['used_joins'] = result
    return myresults


def all_joins(myresults):
    ''' function returns all used joins for a given set of content '''
    result = []
    for element in range(0, len(myresults)):
        sql_join = myresults[element]['sql_joins']
        sql_table_name = myresults[element]['sql_table_name']

        for sql in sql_join:
            if not bool(test_period_appearence(sql)):
                result.append(sql)
            for name in sql_table_name:
                if sql == name:
                    result.append(sql)
        myresults[element]['used_joins'] = result
    return myresults


def all_views(myresults, proj):
    result = []
    for element in range(0, len(myresults)):
        used_joins = myresults[element]['used_joins']
        for join in used_joins:
            if not bool(test_period_appearence(join)):
                result.append(join)
        for file in proj.files():
            path = file.path
            myFile = proj.file(path)
            if myFile.type != 'model':
                for view in myFile.views:
                    if view.sql_table_name.value in used_joins:
                        result.append(view.name)

    myresults[element]['used_view_names'] = result
    return myresults


def match_views_per_query(myresults, proj):
    result = []
    paths = []
    used_joins = myresults['used_joins']
    for join in used_joins:
        if not bool(test_period_appearence(join)):
            result.append(join)
    for file in proj.files():
        path = file.path
        myFile = proj.file(path)
        if myFile.type != 'model':
            for view in myFile.views:
                if view.sql_table_name.value in used_joins:
                    result.append(view.name)
                    paths.append(path)

    myresults['used_view_names'] = result
    myresults['sql_table_paths_'] = paths
    return myresults


def find_unused_views(myresults):
    used_view_names = sorted(myresults['used_view_names'])
    potential_joins = sorted(myresults['potential_join'])
    result = potential_joins

    for view in used_view_names:
        if view in potential_joins:
            result.remove(view)
    myresults['unused_joins'] = result
    return myresults


def match_view_to_dash(content_results, explore_results, sql_table_name, proj):
    tables_in_explore = []

    sql_table_name_ = get_sql_table_name_list(sql_table_name, key=False)

    for content in content_results:
        result = defaultdict(list)
        result['dashboard_id'] = content['dashboard.id']
        result['element_id'] = content['dashboard_element.id']
        result['sql_joins'] = content['sql_joins']
        result['fields_used'] = content['query.formatted_fields']

        result['sql_table_name'] = sql_table_name_

        for explore, tables in explore_results.items():
            if content['query.view'] == explore:
                result['potential_join'] = tables
                tables_in_explore.append(result)
    return tables_in_explore


def main(**kwargs):
    cwd = Path.cwd()
    ini_file = kwargs.get("ini_file")
    config = ConfigParser.RawConfigParser(allow_no_value=True)
    config.read(ini_file)
    project_repo = kwargs.get("project")
    logger.info(f'your project repo is at {project_repo}')
    file_path = kwargs.get("path")
    logger.info(f'your output file is at {file_path}')
    table_mask = kwargs.get("table")
    field_mask = kwargs.get("field")

    create_df.check_ini(ini_file)

    sdk = looker_sdk.init31(config_file=ini_file)

    project = lookml.Project(
        path=project_repo
    )

    content_results = get_dashboards(sdk)
    logger.debug(f'dashboard_response = {content_results}')
    db_response = get_sql_from_elements(sdk, content_results)
    logger.debug(f'db_response = {db_response}')
    explore_results = fetch_view_files(proj=project)
    logger.debug(f'explore results = {explore_results}')

    sql_table_names = get_sql_table_name(proj=project)
    logger.debug(f'sql_table_names = {sql_table_names}')

    combine = match_view_to_dash(
        db_response, explore_results, sql_table_names, proj=project)
    logger.debug(f'combine = {combine}')

    for element in range(0, len(combine)):
        match_join_per_query(combine[element])
        match_views_per_query(combine[element], project)
        find_unused_views(combine[element])

    df = pd.DataFrame(combine)
    del df['sql_table_name']
    del df['potential_join']
    del df['sql_joins']

    if table_mask == None and field_mask == None:
        logger.info('you have not set any field or table filters')

        df.to_csv(f'{file_path}')
        logger.info(df)

    elif table_mask != None:
        logger.info(f'your table filter = {table_mask}')

        mask = df['used_view_names'].apply(lambda x: table_mask in x)
        df = df[mask]
        df.to_csv(f'{file_path}')

        logger.info(df)

    elif field_mask != None:
        logger.info(f'your field filter = {field_mask}')

        mask = df['fields_used'].apply(lambda x: field_mask in x)
        df = df[mask]
        df.to_csv(f'{file_path}')

        logger.info(df)


if __name__ == "__main__":
    main(
        ini_file="/usr/local/google/home/hugoselbie/code_sample/py/projects/ini/k8.ini",
        path='./output.csv',
        project='./tests/test_lookml_files/the_look')



import logging
import coloredlogs
from looker_sdk import models
import looker_sdk

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


def get_unique_groups(
        parsed_yaml: dict) -> list:

    temp_list = []
    for k, v in parsed_yaml.items():
        if 'team' in v.keys():
            team_list = parsed_yaml[k]['team']
            for team in team_list:
                temp_list.append(team)
        else:
            edit_list = parsed_yaml[k]['folder']['team_edit']
            view_list = parsed_yaml[k]['folder']['team_view']
            for team in edit_list:
                temp_list.append(team)
            for team in view_list:
                temp_list.append(team)
    final_return = list(set(temp_list))
    return final_return


def create_group_if_not_exists(
        sdk: looker_sdk,
        group_name: str) -> dict:
    """ Create a Looker Group and add Group attributes

    :param str group_name: Name of a Looker group to create.
    :param dict attributes: Dictionary of attribute names and values (default is None).
    :rtype: Looker Group object.
    """
    # get group if exists
    group = sdk.search_groups(name=group_name)
    if group:
        logger.info(f"Group {group_name} already exists")
        group = group[0]
    else:
        logger.info(f'Creating group "{group_name}"')
        group = sdk.create_group(
            body=models.WriteGroup(
                can_add_to_content_metadata=True,
                name=group_name
            )
        )
    return group


def get_group_metadata(
        sdk: looker_sdk,
        unique_group_list: list) -> list:

    group_metadata = []

    for group_name in unique_group_list:
        group = create_group_if_not_exists(sdk, group_name)
        temp = {}
        temp['group_id'] = group.id
        temp['group_name'] = group.name
        group_metadata.append(temp)

    return group_metadata


def sync_groups(
        sdk: looker_sdk,
        group_metadata_list: list,
        group_name_list: list) -> str:

    all_groups = sdk.all_groups()
    group_dict = {group.name: group.id for group in all_groups}
    # Deleting Standard Groups
    del group_dict['All Users']

    for group_name in group_dict.keys():
        if group_name not in group_name_list:
            sdk.delete_group(group_id=group_dict[group_name])
            logger.info(
                f'deleting group {group_name} to sync with yaml config')

    return 'your groups are in sync with your yaml file'


def search_group_id(sdk: looker_sdk, group_config: dict) -> list:
    group_metadata = []
    for group_name, group_info in group_config.items():
        if 'folder' in group_name:
            logger.debug(group_name)
            folder_name = group_info['folder']['name']
            group = create_group_if_not_exists(sdk, folder_name)
            temp = {}
            temp['group_id'] = group.id
            temp['group_name'] = group.name
            logger.info(f'creating folder {folder_name}')
            try:
                folder = sdk.create_folder(
                    body=models.Folder(
                        name=folder_name,
                        parent_id=1
                    )
                )
                temp['folder_id'] = folder.id
                temp['content_metadata_id'] = folder.content_metadata_id
            except looker_sdk.error.SDKError:
                logger.info('folder has already been created')
                folder = sdk.search_folders(name=folder_name)
                temp['folder_id'] = folder[0]['id']
                temp['content_metadata_id'] = folder[0]['content_metadata_id']
            group_metadata.append(temp)

        else:
            group = create_group_if_not_exists(sdk, group_name)
            logger.info(group)
            temp = {}
            temp['group_id'] = group.id
            temp['group_name'] = group.name
            group_metadata.append(temp)
    return group_metadata



import logging
import time
import coloredlogs
from looker_sdk import models
import looker_sdk

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


def get_user_attribute_metadata(
        sdk: looker_sdk,
        parsed_yaml: dict) -> list:
    response = []
    for k, v in parsed_yaml.items():
        if 'ua' in k:
            response.append(v)
    return response


def sync_user_attributes(
        sdk: looker_sdk,
        ua_metadata: list):
    all_instance_ua = sdk.all_user_attributes()
    all_ua = {ua['name']: ua['id'] for ua in all_instance_ua}


def create_user_attribute(
        sdk: looker_sdk,
        ua_metadata: list):
    for ua in ua_metadata:
        name = ua['name']
        datatype = ua['type']
        value_is_hidden = ua['hidden_value']
        user_view = ua['user_view']
        user_edit = ua['user_edit']

        ua_permissions = models.WriteUserAttribute(
            name=name,
            label=name,
            type=datatype,
            value_is_hidden=value_is_hidden,
            user_can_view=user_view,
            user_can_edit=user_edit
        )

        response = sdk.create_user_attribute(body=ua_permissions)
        print(response)

    # ua_permissions = models.WriteUserAttribute(
    #     name='foo',
    #     type='string',
    #     value_is_hidden=false,
    #     user_can_view=true,
    #     user_can_edit=false)
    # pass



import logging
import sys
import time
import json
import coloredlogs
from pprint import pprint
import looker_sdk
from looker_sdk import models

logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


def get_role_metadata(
        parsed_yaml: dict) -> list:
    role_permission = []
    del parsed_yaml['role_admin']
    for group_name, group_info in parsed_yaml.items():
        if 'role' in group_name:
            role_name = group_info['role']
            temp = {}
            if 'permissions' in group_info.keys():
                temp['role_name'] = role_name
                temp['permission'] = group_info['permissions']
                temp['model_set_value'] = group_info['model_set']
                temp['teams'] = group_info['team']
                role_permission.append(temp)
            else:
                temp['role_name'] = role_name
                role_permission.append(temp)

    logger.debug(role_permission)
    return role_permission


def sync_permission_set(
        sdk: looker_sdk,
        all_permission_sets: list,
        permission_set_list: list):

    permissions_dict = {p.name: p.id for p in all_permission_sets}
    permissions_dict.pop('Admin')
    yaml_permissions = [role.get('name') for role in permission_set_list]

    for permission_set_name in permissions_dict.keys():

        if permission_set_name not in yaml_permissions:
            permission_id = sdk.search_permission_sets(
                name=permission_set_name)[0].id
            sdk.delete_permission_set(permission_set_id=permission_id)


def create_permission_set(
        sdk: looker_sdk,
        permission_set_list: list):

    final_response = []
    for permission in permission_set_list:
        permission_set_name = permission.get('role_name')
        permissions = permission.get('permission')
        body = models.WritePermissionSet(
            name=permission_set_name.lower(),
            permissions=permissions
        )
        try:
            perm = sdk.create_permission_set(
                body=body
            )
        except looker_sdk.error.SDKError:
            perm = sdk.search_permission_sets(name=permission_set_name)[0]
            pid = perm.id
            perm = sdk.update_permission_set(permission_set_id=pid, body=body)

        temp = {}
        temp['name'] = perm.name
        temp['pid'] = perm.id
        final_response.append(temp)

    logger.debug(final_response)
    return final_response


def create_model_set(
        sdk: looker_sdk,
        model_set_list: list) -> list:

    final_response = []
    for model in model_set_list:
        model_sets = model.get('model_set_value')
        model_set_name = model.get('role_name')
        body = models.WriteModelSet(
            name=model_set_name.lower(), models=model_sets)
        try:
            model = sdk.create_model_set(body=body)
        except looker_sdk.error.SDKError as modelerror:
            logger.info(modelerror.args[0])
            model = sdk.search_model_sets(name=model_set_name)[0]
            model_set_id = model.id
            model = sdk.update_model_set(model_set_id=model_set_id, body=body)
        temp = {}
        temp['model_set_name'] = model.name
        temp['model_set_id'] = model.id
        final_response.append(temp)
    logger.info(final_response)
    return final_response


def sync_model_set(
        sdk: looker_sdk,
        all_model_sets: list,
        model_set_list: list):

    model_sets_dict = {p.name: p.id for p in all_model_sets}
    model_sets_dict.pop('All')
    yaml_model = [role.get('model_set_name') for role in model_set_list]

    for model_set_name in model_sets_dict.keys():

        if model_set_name not in yaml_model:
            model_id = sdk.search_model_sets(
                name=model_set_name)[0].id
            sdk.delete_model_set(model_set_id=model_id)


def create_roles(
        sdk: looker_sdk,
        all_model_sets: list,
        all_permission_sets: list,
        role_metadata_list: list) -> list:

    model_set_dict = {model.name: model.id for model in all_model_sets}
    permission_set_dict = {perm.name: perm.id for perm in all_permission_sets}

    role_output = []
    for role in role_metadata_list:
        role_name = role.get('role_name')
        permission_set_id = permission_set_dict.get(role_name.lower())
        model_set_id = model_set_dict.get(role_name.lower())
        body = models.WriteRole(
            name=role_name,
            permission_set_id=permission_set_id,
            model_set_id=model_set_id
        )
        try:
            role = sdk.create_role(
                body=body
            )
        except looker_sdk.error.SDKError as roleerror:
            logger.debug(roleerror)
            role_id = sdk.search_roles(name=role_name)[0].id
            role = sdk.update_role(role_id=role_id, body=body)
        temp = {}
        temp['role_id'] = role.id
        temp['role_name'] = role_name
        role_output.append(temp)
    logger.info(role_output)
    return role_output


def sync_roles(
        sdk: looker_sdk,
        all_roles: list,
        role_metadata_list: list):

    all_role_dict = {role.name: role.id for role in all_roles}
    all_role_dict.pop('Admin')
    yaml_role = [role.get('role_name') for role in role_metadata_list]

    for role_name in all_role_dict.keys():
        if role_name not in yaml_role:
            role_id = sdk.search_roles(name=role_name)[0].id
            sdk.delete_role(role_id=role_id)


def set_role(
        role_id: str,
        sdk: looker_sdk,
        group_id: list) -> str:
    try:
        sdk.set_role_groups(role_id, group_id)
        return logger.info(f'attributing {group_id} permissions on instance')
    except looker_sdk.error.SDKError:
        return logger.info('something went wrong')


def attach_role_to_group(
    sdk: looker_sdk,
    role_metadata: list,
        created_role_metadata: list,
        all_roles: list):

    all_groups = sdk.all_groups()
    role_dict = {role.name: role.id for role in all_roles}
    group_dict = {group.name: group.id for group in all_groups}

    for role in role_metadata:
        teams = role.get('teams')
        role_id = role_dict.get(role.get('role_name'))
        group_id_list = []
        for team in teams:
            group_id = group_dict.get(team)
            group_id_list.append(group_id)
        set_role(role_id=role_id, group_id=group_id_list, sdk=sdk)

# def folder_output(
#         sdk: looker_sdk,
#         group_metadata: list,
#         group_config: dict) -> list:
#     response = []
#     for group_name, group_info in group_config.items():
#         folder_info = group_info.get('folder', None)

#         if folder_info:
#             content_metadata_id = sdk.search_folders(
#                 name=folder_info['name'])
#             for group in group_metadata:
#                 folder_provision = {}
#                 folder_provision['name'] = folder_info['name']
#                 folder_provision['cmi'] = content_metadata_id[0].content_metadata_id
#                 # folder_provision['folder_id'] = folder_info['folder_id']
#                 if folder_info['team_edit'] in group.values():
#                     folder_provision['group_id'] = group['group_id']
#                     folder_provision['content_metadata_id'] = group['content_metadata_id']

#                     folder_provision['team_info'] = {}
#                     folder_provision['team_info']['folder_id'] = group['folder_id']
#                     folder_provision['team_info']['group_id'] = group['group_id']
#                     folder_provision['team_info']['group_name'] = group['group_name']
#                     folder_provision['team_info']['permission'] = 'edit'
#                     response.append(folder_provision)
#                 else:
#                     for team in folder_info['team_view']:
#                         if team in group.values():
#                             folder_provision['team_info'] = {}
#                             folder_provision['group_id'] = group['group_id']
#                             # folder_provision['content_metadata_id'] = group['content_metadata_id']
#                             folder_provision['content_metadata_id'] = group['content_metadata_id']
#                             folder_provision['team_info']['folder_id'] = group['folder_id']
#                             folder_provision['team_info']['group_id'] = group['group_id']
#                             folder_provision['team_info']['group_name'] = group['group_name']
#                             folder_provision['team_info']['permission'] = 'view'
#                             response.append(folder_provision)

#     return response


# def ancestor_folder_change():
#     ancestors = sdk.folder_ancestors(folder_id=folder_id)
#     pass


# def provision_folders_with_group_access(
#         sdk: looker_sdk,
#         folder_permission_metadata: list) -> str:

#     for access_item in folder_permission_metadata:

#         content_metadata_id = access_item["cmi"]

#         # check for existing access to the folder
#         content_metadata_accesses = {
#             access.group_id: access
#             for access in sdk.all_content_metadata_accesses(
#                 content_metadata_id=content_metadata_id)}

#         group_id = access_item['team_info']['group_id']
#         permission = access_item['team_info']['permission']
#         if group_id in content_metadata_accesses.keys():
#             current_access = content_metadata_accesses.get(group_id)

#             if current_access.permission_type.value == permission:
#                 logger.info(
#                     f'--> Group {group_id} already has access, no changes made.')

#             else:
#                 # don't want to inherit access from parent folders
#                 sdk.update_content_metadata(
#                     content_metadata_id=content_metadata_id,
#                     body={'inherits': False}
#                 )
#                 sdk.update_content_metadata_access(
#                     content_metadata_access_id=current_access.id,
#                     body=models.ContentMetaGroupUser(
#                         content_metadata_id=content_metadata_id,
#                         permission_type=permission,
#                         group_id=group_id
#                     ))

#                 logging.info(
#                     f'--> Changed permission type to {permission}.')

#         # no existing access
#         # create from scratch
#         else:
#             # don't want to inherit access from parent folders
#             sdk.update_content_metadata(
#                 content_metadata_id=content_metadata_id,
#                 body={'inherits': False}
#             )

#             sdk.create_content_metadata_access(
#                 body=models.ContentMetaGroupUser(
#                     content_metadata_id=content_metadata_id,
#                     permission_type=permission,
#                     group_id=group_id
#                 )
#             )
#             logger.info(
#                 f'--> Sucessfully permissioned group {group_id} {permission} access.')


# def remove_all_user_group(
#         sdk: looker_sdk,
#         folder_permission_metadata: list):

#     # remove parent Shared group instance access
#     try:
#         sdk.update_content_metadata_access(
#             content_metadata_access_id=1,
#             body=models.ContentMetaGroupUser(
#                 permission_type='view',
#                 content_metadata_id=1,
#                 group_id=1
#             )
#         )
#     except looker_sdk.error.SDKError:
#         logger.info('All Users group already configured')
#     clean = list()
#     for avt in folder_permission_metadata:
#         temp = {}
#         temp['name'] = avt['name']
#         temp['cmi'] = avt['cmi']
#         clean.append(temp)
#     res_list = [i for n, i in enumerate(clean) if i not in clean[n + 1:]]

#     for access_item in res_list:
#         time.sleep(1)

#         content_metadata_id = access_item["cmi"]

#         # check for existing access to the folder
#         content_metadata_accesses = {
#             access.group_id: access
#             for access in sdk.all_content_metadata_accesses(
#                 content_metadata_id=content_metadata_id)}

#         for id, value in content_metadata_accesses.items():
#             logger.debug(f'Checking item {value.id}')
#             if value.group_id == 1:
#                 cmaid = value['id']
#                 sdk.delete_content_metadata_access(
#                     content_metadata_access_id=cmaid)
#                 break


def main(**kwargs):
    """ Sync Looker group membership, group attributes, roles, and content access

     This method creates groups with the specified membership and user attributes,
     assigns the groups to the specified roles, and permissions access to the
     specified folders. Note that each group only has one of (1) role, (2) attributes
     and (3) folder access.

     :param dict group_config: Mapping from Looker group name to group metadata.
     :param dict ldap_membership: Mapping from an LDAP group name to a list of member LDAPs.
     :rtype: None
     """
    div = '--------------------------------------'

    ini_file = kwargs.get("ini_file")
    yaml_path = kwargs.get("yaml_path")
    logger.info(div)
    logger.info('parsing yaml file')
    group_config = read_provision_yaml(yaml_path)
    sdk = looker_sdk.init31(config_file=ini_file)

    group_metadata = search_group_id(sdk, group_config)

    # collect content access information
    logger.info(div)
    logger.info('Collecting content access information...')
    folder_permission_metadata = folder_output(sdk=sdk,
                                               group_metadata=group_metadata, group_config=group_config)

    # update content access
    logger.info(div)
    logger.info('Adding content access...')
    provision_folders_with_group_access(
        sdk, folder_permission_metadata=folder_permission_metadata)

    # create roles
    logger.info(div)
    logging.info('Creating Roles')
    role_metadata = create_role_mapping(group_config=group_config)
    role_create = create_roles(sdk=sdk, role_mapping=role_metadata)
    attach_role_to_group(sdk=sdk, role_metadata=role_create,
                         group_config=group_config)

    # removing all user group from folders
    logger.info(div)
    logger.info('Removing all user group from folders')
    remove_all_user_group(
        sdk=sdk, folder_permission_metadata=folder_permission_metadata)


if __name__ == "__main__":
    main(
        ini_file='/usr/local/google/home/hugoselbie/code_sample/py/ini/k8.ini',
        yaml_path='/usr/local/google/home/hugoselbie/code_sample/py/clients/snap/okta_groups/snap_group_mapping.yaml')



import logging
import time
import coloredlogs
from looker_sdk import models
import looker_sdk

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


def get_content_access_metadata(
        sdk: looker_sdk,
        parsed_yaml: dict) -> list:
    response = []
    for folder_name, folder_info in parsed_yaml.items():
        folder_provision = {}
        if 'folder' in folder_name:
            fname = folder_info['folder']['name']
            fmetadata = sdk.search_folders(name=fname)
            folder_provision['cmi'] = fmetadata[0].content_metadata_id
            folder_provision['name'] = fname

            edit_group = folder_info['folder']['team_edit']
            view_group = folder_info['folder']['team_view']
            temp = []
            for group in edit_group:
                temp_dict = {}
                egmetadata = sdk.search_groups(name=group)
                temp_dict['name'] = group
                temp_dict['id'] = egmetadata[0].id
                temp_dict['permission'] = 'edit'
                temp.append(temp_dict)

            for group in view_group:
                temp_dict = {}
                vgmetadata = sdk.search_groups(name=group)
                temp_dict['name'] = group
                temp_dict['id'] = vgmetadata[0].id
                temp_dict['permission'] = 'view'
                temp.append(temp_dict)
            folder_provision['group_permissions'] = temp
            response.append(folder_provision)
    return response


def ancestor_folder_change():
    pass
    # ancestors = sdk.folder_ancestors(folder_id=folder_id)


def provision_folders_with_group_access(
        sdk: looker_sdk,
        content_access_metadata_list: list) -> str:

    for access_item in content_access_metadata_list:

        content_metadata_id = access_item["cmi"]

        # check for existing access to the folder
        content_metadata_accesses = {
            access.group_id: access
            for access in sdk.all_content_metadata_accesses(
                content_metadata_id=content_metadata_id)}

        # sync content_metadata back to yaml file
        true_group_id = [access_item['group_permissions'][elem]['id']
                         for elem in range(0, len(access_item['group_permissions']))]
        for group_id in content_metadata_accesses.keys():
            if group_id not in true_group_id:
                delete_cmi = content_metadata_accesses.get(group_id).id
                sdk.delete_content_metadata_access(
                    content_metadata_access_id=delete_cmi)

        for elem in range(0, len(access_item['group_permissions'])):
            group_id = access_item['group_permissions'][elem]['id']
            permission = access_item['group_permissions'][elem]['permission']
            if group_id in content_metadata_accesses.keys():
                current_access = content_metadata_accesses.get(group_id)

                if current_access.permission_type == permission:
                    logger.info(
                        f'--> Group {group_id} already has access, no changes made.')

                else:
                    # don't want to inherit access from parent folders
                    sdk.update_content_metadata(
                        content_metadata_id=content_metadata_id,
                        body=models.WriteContentMeta(inherits=False)
                    )
                    try:
                        sdk.update_content_metadata_access(
                            content_metadata_access_id=current_access.id,
                            body=models.ContentMetaGroupUser(
                                content_metadata_id=content_metadata_id,
                                permission_type=permission,
                                group_id=group_id
                            ))
                    except looker_sdk.error.SDKError as foldererror:
                        logger.debug(foldererror.args[0])

                    logging.info(
                        f'--> Changed permission type to {permission}.')

            # no existing access
            # create from scratch
            else:
                # don't want to inherit access from parent folders
                sdk.update_content_metadata(
                    content_metadata_id=content_metadata_id,
                    body=models.WriteContentMeta(inherits=False)
                )

                sdk.create_content_metadata_access(
                    body=models.ContentMetaGroupUser(
                        content_metadata_id=content_metadata_id,
                        permission_type=permission,
                        group_id=group_id
                    )
                )
                logger.info(
                    f'--> Sucessfully permissioned group {group_id} {permission} access.')


def remove_all_user_group(
        sdk: looker_sdk,
        content_access_metadata_list: list):

    # remove parent Shared group instance access
    try:
        sdk.update_content_metadata_access(
            content_metadata_access_id=1,
            body=models.ContentMetaGroupUser(
                permission_type='view',
                content_metadata_id=1,
                group_id=1
            )
        )
    except looker_sdk.error.SDKError:
        logger.info('All Users group already configured')
    clean = list()
    for avt in content_access_metadata_list:
        temp = {}
        temp['name'] = avt['name']
        temp['cmi'] = avt['cmi']
        clean.append(temp)
    res_list = [i for n, i in enumerate(clean) if i not in clean[n + 1:]]

    for access_item in res_list:
        time.sleep(1)

        content_metadata_id = access_item["cmi"]

        # check for existing access to the folder
        content_metadata_accesses = {
            access.group_id: access
            for access in sdk.all_content_metadata_accesses(
                content_metadata_id=content_metadata_id)}

        for id, value in content_metadata_accesses.items():
            logger.debug(f'Checking item {value.id}')
            if value.group_id == 1:
                cmaid = value.id
                sdk.delete_content_metadata_access(
                    content_metadata_access_id=cmaid)
                break



import looker_sdk
from looker_sdk import models
import coloredlogs
import logging

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')


def get_unique_folders(
        sdk: looker_sdk,
        parsed_yaml: dict) -> list:

    folder_metadata = []
    for k, v in parsed_yaml.items():
        if 'folder' in k:
            folder_name = parsed_yaml[k]['folder']['name']
            folder_metadata.append(folder_name)

    return folder_metadata


def create_folder_if_not_exists(
        sdk: looker_sdk,
        folder_name: str) -> dict:

    folder = sdk.search_folders(name=folder_name)
    if folder:
        logger.info(f"Folder {folder_name} already exists")
        folder = folder[0]
    else:
        logger.info(f'Creating folder "{folder_name}"')
        folder = sdk.create_folder(
            body=models.CreateFolder(
                name=folder_name,
                parent_id="1"
            )
        )
    return folder


def get_folder_metadata(
        sdk: looker_sdk,
        unique_folder_list: list) -> list:

    folder_metadata = []

    for folder_name in unique_folder_list:
        folder = create_folder_if_not_exists(sdk, folder_name)
        temp = {}
        temp['folder_id'] = folder.id
        temp['folder_name'] = folder.name
        temp['content_metadata_id'] = folder.content_metadata_id
        folder_metadata.append(temp)

    return folder_metadata


def sync_folders(
        sdk: looker_sdk,
        folder_metadata_list: list,
        folder_name_list: list) -> str:

    all_folders = sdk.all_folders()
    folder_dict = {
        folder.name: folder.id for folder in all_folders if folder.parent_id == "1"}

    for folder_name in folder_dict.keys():
        if folder_name not in folder_name_list:
            sdk.delete_folder(folder_id=folder_dict[folder_name])
            logger.info(
                f'deleting folder {folder_name} to sync with yaml config')

    return 'your folders are in sync with your yaml file'



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lmanage',
 'lmanage.create_folder_permissions',
 'lmanage.create_user_attribute_permissions',
 'lmanage.create_user_permissions',
 'lmanage.utils',
 'lmanage.utils.old_files']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'coloredlogger>=1.3.12,<2.0.0',
 'coloredlogs>=15.0,<16.0',
 'debugpy>=1.3.0,<2.0.0',
 'flake8>=4.0.1,<5.0.0',
 'icecream>=2.1.0,<3.0.0',
 'ipython>=7.20.0,<8.0.0',
 'lkml>=1.1.1,<2.0.0',
 'lookml>=3.0.3,<4.0.0',
 'pandas>=1.2.2,<2.0.0',
 'pylint>=2.9.5,<3.0.0',
 'pynvim>=0.4.3,<0.5.0',
 'pytest-mock>=3.5.1,<4.0.0',
 'snoop>=0.3.0,<0.4.0',
 'sqlparse>=0.4.1,<0.5.0',
 'tabulate>=0.8.8,<0.9.0',
 'verboselogs>=1.7,<2.0']

entry_points = \
{'console_scripts': ['lmanage = lmanage.cli:lmanage']}

setup_kwargs = {
    'name': 'lmanage',
    'version': '0.2.6.2',
    'description': "LManage is a collection of useful tools for Looker admins to help curate and cleanup content and it's associated source LookML.",
    'long_description': '# Lmanage\n## What is it.\nLManage is a collection of useful tools for [Looker](https://looker.com/) admins to help curate and cleanup content and it\'s associated source [LookML](https://docs.looker.com/data-modeling/learning-lookml/what-is-lookml).\n\n## How do i Install it.\nLmanage can be found on [pypi](#).\n```\npip install lmanage\n```\n\n## How do I Use it.\n### Commands\nLManage will ultimately will have many different commands as development continues \n| Status  | Command    | Rationale                                                                                                                                                                                            |\n|---------|------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|\n| Live    | mapview    | Find the LookML fields and tables that are associated with a piece of Looker content                                                                                                                 |\n| Planned | removeuser | Based on last time logged in, prune Looker users to ensure a performant, compliant Looker instance                                                                                                   |\n| Planned | dcontent   | Iterate through an input of content, delete content and back it up using [gzr](https://github.com/looker-open-source/gzr) for easy restoration                                                                                               |\n| Planned | bcontent   | Iterate through all broken content (using content validator) and email a customized message to each dashboard owner                                                                                  |\n| Planned | scoper     | Takes in a model file, elminates the * includes, iterate through the explores and joins and creates a fully scoped model include list for validation performance and best practice code organization |\n\n#### help and version\n```\nlmanage --help\nUsage: lmanage [OPTIONS] COMMAND [ARGS]...\n\nOptions:\n  --version  Show the version and exit.\n  --help     Show this message and exit.\n\nCommands:\n  mapview\n```\n#### mapview\nThe mapview command will find the etymology of the content on your dashboard, exporting a CSV that looks like [this](https://docs.google.com/spreadsheets/d/1TzeJW46ml0uzO9RdLOOLxwtvUWjhmZxoa-xq4pbznV0/edit?resourcekey=0-xbWC87hXYFNgy1As06NncA#gid=900312158).\n\n##### example usage\n`lmanage mapview --path ./output/my_output.csv --ini-file ~/py/projects/ini/k8.ini --project /test_lookml_files/the_look -table "order_items"`\n##### flags\n- **path** (`--path`, `-fp`) This is the path where you would like the outputfile for your returned dataset to be. \n- **ini-file** (`--ini-file`, `-i`) This is the file path to the ini file that you\'ve created to use the Looker SDK\n```\n#example Ini file\n[Looker_Instance]\nbase_url=https://looker-dev.company.com:19999 (or 443 if hosted in GCP)\nclient_id=abc\nclient_secret=xyz\nverify_ssl=True\n```\n- **project** (`--project`, `-p`) This is the file path to your local project of LookML files that lmanage will scan to associate connections between your Looker content and LookML\nLmanage can either return a full dataset of all content mapping, or a prefiltered dataset with all content associated with a specific table or field.\n- **table** (`--table`, `-t`) **Optional** Expecting input of lookml view name\n- **field** (`--field`, `-f`) **Optional** Expecting input of fully scoped LookML field name e.g. viewname.fieldname \n- **level** (`--level`, `-l`) **Optional** Set this flag to DEBUG to receive expanded results in stdout for debugging  \n\n\n![](./images/mapview_walkthru.jpeg)\n\n\n## Fields Returneds\n\n- **dashboard_id**: the id of the looker dashboard \t\n- **element_id**: the id of the visualization element on the looker dashboard\t\n- **sql_joins**: the joins used in a query grouped by element id\t\n- **fields_used**: the fields used by the query grouped by element id\n- **sql_table_name**: the underlying sql value being referenced at the view level of the lookml (assuming the view is standard)\t\n- **lookml_file_name**: the physical file in which the view files reside\n- **potential_join**: for the explore that powers the element query: what are all the potential joins available\t\n- **used_joins**: joins used by the underlying queries obtained by parsing sql of query \t\n- **used_view_names**: views that are used by each query grouped by element_id\t\n- **unused_joins**: views that are unused by the specific query of the dashboard element\n\n##### n.b.\n**Multi Project Usage**\nDashboards can hold tiles from multiple projects, in this case if you create one local folder of lookml see example below, then pass the value of that one meta folder the the `--project` flag. Doing this will enable the underlying LookML parsing engine driven by [pyLookML](https://github.com/llooker/pylookml) to iterate over all the relevant files and find the appropriate cross project matches.\n\n```\n├── test_lookml_files\n    │    ├── dashboards\n    │    │   ├── brand_lookup.dashboard.lookml\n    │    │   ├── business_pulse.dashboard.lookml\n    │    │   ├── customer_lookup.dashboard.lookml\n    │    ├── models_proj1\n    │    │   └── thelook.model.lkml\n    │    ├── models_proj2\n    │    │   └── thelook_redshift.model.lkml\n    │    ├── view_proj1\n    │    │   ├── 01_order_items.view.lkml\n    │    │   ├── 02_users.view.lkml\n    │    │   ├── 03_inventory_items.view.lkml\n    │    │   ├── 04_products.view.lkml\n    │    │   ├── 05_distribution_centers.view.lkml\n    │    └── view_proj2\n    │        ├── 01_order_items.view.lkml\n    │        ├── 02_users.view.lkml\n    │        ├── 03_inventory_items.view.lkml\n    │        ├── 04_products.view.lkml\n    │        ├── 05_distribution_centers.view.lkml\n    │        ├── 11_order_facts.view.lkml\n    │        ├── 12_user_order_facts.view.lkml\n    │        ├── 13_repeat_purchase_facts.view.lkml\n    │        ├── 22_affinity.view.lkml\n    │        ├── 25_trailing_sales_snapshot.view.lkml\n    │        ├── 51_events.view.lkml\n    │        ├── explores.lkml\n    │        └── test_ndt.view.lkml.\n```\n\n\n\n**This is not an officially supported Google Product.**\n',
    'author': 'hselbie',
    'author_email': 'hselbie@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/looker-open-source/lmanage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.9',
}


setup(**setup_kwargs)



import logging
import coloredlogs
from looker_sdk import models
import looker_sdk
import parse_yaml

logger = logging.getLogger(__name__)
coloredlogs.install(level='DEBUG')
path = '/usr/local/google/home/hugoselbie/code_sample/py/lmanage/tests/example_yamls/fullinstance.yaml'
data = parse_yaml.Yaml(yaml_path=path)
dd = data.read_provision_yaml()


def get_unique_folders(
        sdk: looker_sdk,
        parsed_yaml: dict) -> list:

    folder_metadata = []
    for k, v in parsed_yaml.items():
        if 'folder' in k:
            temp = {}
            folder_name = parsed_yaml['folder']['name']

            try:
                folder = sdk.create_folder(
                    body=models.Folder(
                        name=folder_name,
                        parent_id=1
                    )
                )
                temp['folder_id'] = folder.id
                temp['content_metadata_id'] = folder.content_metadata_id
            except looker_sdk.error.SDKError:
                logger.info('folder has already been created')
                folder = sdk.search_folders(name=folder_name)
                temp['folder_id'] = folder[0]['id']
                temp['content_metadata_id'] = folder[0]['content_metadata_id']
            folder_metadata.append(temp)
    return folder_metadata


x = get_unique_folders(sdk=sdk, parsed_yaml=dd)
print(x)



"""
Copyright 2021 Google LLC

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import itertools
import sqlparse

from sqlparse.sql import IdentifierList, Identifier
from sqlparse.tokens import Keyword, DML


def is_subselect(parsed):
    if not parsed.is_group:
        return False
    for item in parsed.tokens:
        if item.ttype is DML and item.value.upper() == 'SELECT':
            return True
    return False


def extract_from_part(parsed):
    from_seen = False
    for item in parsed.tokens:
        if item.is_group:
            for x in extract_from_part(item):
                yield x
        if from_seen:
            if is_subselect(item):
                for x in extract_from_part(item):
                    yield x
            elif item.ttype is Keyword and item.value.upper() in ['ORDER', 'GROUP', 'BY', 'HAVING', 'GROUP BY']:
                from_seen = False
                StopIteration
            else:
                yield item
        if item.ttype is Keyword and item.value.upper() == 'FROM':
            from_seen = True


def extract_table_identifiers(token_stream):
    for item in token_stream:
        if isinstance(item, IdentifierList):
            for identifier in item.get_identifiers():
                value = identifier.value.replace('"', '').lower()
                yield value
        elif isinstance(item, Identifier):
            value = item.value.replace('"', '').lower()
            yield value


def extract_tables(sql):
    # let's handle multiple statements in one sql string
    extracted_tables = []
    statements = list(sqlparse.parse(sql))
    for statement in statements:
        if statement.get_type() != 'UNKNOWN':
            stream = extract_from_part(statement)
            extracted_tables.append(
                set(list(extract_table_identifiers(stream))))
    rough_tables = list(itertools.chain(*extracted_tables))
    rough_tables = [elem.split(' ') for elem in rough_tables]
    clean_list = []
    for elem in rough_tables:
        select = []
        for i in elem:
            if i:
                select.append(i)

        if len(select) < 4:
            clean_list.append(select)

    clean_table = [elem[0].strip('\n')
                   for elem in clean_list if not elem[0].startswith('z__')]
    # print(clean_table)
    return clean_table


if __name__ == '__main__':
    sql = """WITH order_user_sequence_facts AS (select oi.user_id,oi.id as order_id,row_number() over(partition by oi.user_id order by oi.created_at asc ) as order_sequence,
        oi.created_at,
        MIN(oi.created_at) OVER(PARTITION BY oi.user_id) as first_ordered_date,
        LAG(oi.created_at) OVER (PARTITION BY oi.user_id ORDER BY oi.created_at asc) as previous_order_date,
        LEAD(oi.created_at) OVER(partition by oi.user_id ORDER BY oi.created_at) as next_order_date,
        DATEDIFF(DAY,CAST(oi.created_at as date),CAST(LEAD(oi.created_at) over(partition by oi.user_id ORDER BY oi.created_at) AS date)) as repurchase_gap
      from order_items oi
 )
SELECT * FROM (
SELECT *, DENSE_RANK() OVER (ORDER BY z___min_rank) as z___pivot_row_rank, RANK() OVER (PARTITION BY z__pivot_col_rank ORDER BY z___min_rank) as z__pivot_col_ordering, CASE WHEN z___min_rank = z___rank THEN 1 ELSE 0 END AS z__is_h
ighest_ranked_cell FROM (
SELECT *, MIN(z___rank) OVER (PARTITION BY "order_user_sequence_facts.created_at_month") as z___min_rank FROM (
SELECT *, RANK() OVER (ORDER BY "order_user_sequence_facts.created_at_month" DESC, z__pivot_col_rank) AS z___rank FROM (
SELECT *, DENSE_RANK() OVER (ORDER BY "users.gender" NULLS LAST) AS z__pivot_col_rank FROM (
SELECT
    users.gender  AS "users.gender",
        (TO_CHAR(DATE_TRUNC('month', CONVERT_TIMEZONE('UTC', 'America/New_York', order_user_sequence_facts.created_at )), 'YYYY-MM')) AS "order_user_sequence_facts.created_at_month",
    COUNT(DISTINCT order_user_sequence_facts.user_id ) AS "order_user_sequence_facts.count"
FROM public.order_items  AS order_items
INNER JOIN public.users  AS users ON order_items.user_id = users.id
LEFT JOIN public.inventory_items  AS inventory_items ON inventory_items.id = order_items.inventory_item_id
LEFT JOIN order_user_sequence_facts ON users.id = order_user_sequence_facts.user_id
WHERE (order_user_sequence_facts.order_sequence = 1
    )
GROUP BY
    (DATE_TRUNC('month', CONVERT_TIMEZONE('UTC', 'America/New_York', order_user_sequence_facts.created_at ))),
    1) ww
) bb WHERE z__pivot_col_rank <= 16384
) aa
) xx
) zz
 WHERE (z__pivot_col_rank <= 50 OR z__is_highest_ranked_cell = 1) AND (z___pivot_row_rank <= 500 OR z__pivot_col_ordering = 1) ORDER BY z___pivot_row_rank
    """

    print(extract_tables(sql))
    # tables = ', '.join(extract_tables(sql))
    # print('Tables: {}'.format(tables))



import logging
import coloredlogs
import looker_sdk
from .create_folder_permissions import create_folder_permissions as cfp
from .create_user_permissions import user_permission as up
from .create_user_attribute_permissions import create_ua_permissions as cuap
from .utils import group_config as gc
from .utils import folder_config as fc
from .utils import parse_yaml as py


logger = logging.getLogger(__name__)
coloredlogs.install(level='INFO')


def main(**kwargs):
    div = '--------------------------------------'

    ini_file = kwargs.get("ini_file")
    yaml_path = kwargs.get("yaml_config_path")
    logger.info(div)
    logger.info('parsing yaml file')
    yaml = py.Yaml(yaml_path=yaml_path)
    instance_config = yaml.read_provision_yaml()
    if ini_file:
        sdk = looker_sdk.init31(config_file=ini_file)
    else:
        sdk = looker_sdk.init31()

###############################################################
# Group Config ################################################
###############################################################
    # # FIND UNIQUE GROUPS FROM YAML FILE
    unique_group_names = gc.get_unique_groups(parsed_yaml=instance_config)
    logger.info(div)

    # # CREATE NEW GROUPS
    group_metadata = gc.get_group_metadata(
        sdk=sdk, unique_group_list=unique_group_names)
    logger.info(div)

    # # DELETE ALL GROUPS THAT DO NOT MATCH WITH YAML
    gc.sync_groups(group_name_list=unique_group_names,
                   group_metadata_list=group_metadata, sdk=sdk)
    logger.info(div)

###############################################################
# Role Config ################################################
###############################################################
    # # FIND UNIQUE ROLES
    role_metadata = up.get_role_metadata(parsed_yaml=instance_config)

    # # CREATE PERMISSION SETS
    permission_set_metadata = up.create_permission_set(
        sdk=sdk, permission_set_list=role_metadata)

    # # SYNC PERMISSION SETS
    all_permission_sets = sdk.all_permission_sets()
    up.sync_permission_set(
        sdk=sdk, all_permission_sets=all_permission_sets,
        permission_set_list=permission_set_metadata)

    # # CREATE MODEL SETS
    model_set_metadata = up.create_model_set(
        sdk=sdk, model_set_list=role_metadata)

    # # SYNC MODEL SETS
    all_model_sets = sdk.all_model_sets()
    up.sync_model_set(sdk=sdk,
                      all_model_sets=all_model_sets,
                      model_set_list=model_set_metadata)

    # # CREATE NEW ROLES
    created_role_metadata = up.create_roles(sdk=sdk,
                                            all_model_sets=all_model_sets,
                                            all_permission_sets=all_permission_sets,
                                            role_metadata_list=role_metadata)

    # # SYNC ROLES
    all_roles = sdk.all_roles()
    up.sync_roles(
        sdk=sdk,
        all_roles=all_roles,
        role_metadata_list=created_role_metadata
    )
    # # ATTACH ROLES TO TEAM GROUPS
    up.attach_role_to_group(sdk=sdk,
                            role_metadata=role_metadata,
                            created_role_metadata=created_role_metadata,
                            all_roles=all_roles)

###############################################################
# Folder Config ################################################
###############################################################
    # # FIND UNIQUE FOLDERS
    unique_folder_names = fc.get_unique_folders(
        sdk=sdk, parsed_yaml=instance_config)
    logger.info(div)

    # # CREATE NEW FOLDERS
    folder_metadata = fc.get_folder_metadata(
        sdk=sdk, unique_folder_list=unique_folder_names)
    logger.info(div)

    # # CONFIGURE FOLDERS WITH EDIT AND VIEW ACCESS
    content_access_metadata = cfp.get_content_access_metadata(
        sdk=sdk,
        parsed_yaml=instance_config)

    # # ADD AND SYNC CONTENT VIEW ACCESS WITH YAML
    cfp.provision_folders_with_group_access(
        sdk=sdk, content_access_metadata_list=content_access_metadata)

    cfp.remove_all_user_group(
        sdk=sdk, content_access_metadata_list=content_access_metadata)
    # # DELETE ALL FOLDERS THAT DON'T MATCH WITH YAML
    fc.sync_folders(sdk=sdk, folder_metadata_list=folder_metadata,
                    folder_name_list=unique_folder_names)

    logger.info(div)

################################################################
## User Attribute Config #######################################
################################################################
#    # # FIND UNIQUE USER ATTRIBUTES
#    ua_metadata = cuap.get_user_attribute_metadata(
#        sdk=sdk,
#        parsed_yaml=instance_config)

#    # # CREATE NEW FOLDERS

#    cuap.create_user_attribute(
#        sdk=sdk,
#        ua_metadata=ua_metadata
#    )

#    # # CONFIGURE FOLDERS WITH EDIT AND VIEW ACCESS

#    # # ADD AND SYNC CONTENT VIEW ACCESS WITH YAML

#    # # DELETE ALL FOLDERS THAT DON'T MATCH WITH YAML


if __name__ == "__main__":
    YP = ('/usr/local/google/home/hugoselbie/code_sample/py/'
          'lmanage/tests/example_yamls/fullinstance.yaml')
    IP = ('/usr/local/google/home/hugoselbie/code_sample/py/ini/k8.ini')

    main(
        ini_file=IP,
        yaml_config_path=YP)
    logger.info('I have finished')
